@@ -4,9 +4,16 @@
 # 3) we can import it into your module module
 # https://stackoverflow.com/questions/458550/standard-way-to-embed-version-into-python-package
 
-__version__ = '1.43.7'
+__version__ = '1.44.3'
 
 release_notes = {
+    '1.44.3': """
+    Splitting the legacy GUI into a separate package to break our dependency on PyQt5.
+    If you still really need it, you can find it at https://pypi.org/project/rlbot_legacy_gui/
+    
+    Also:
+    - More elegant handling of Continue and Spawn scenarios.
+    """,
     '1.43.7': """
     Bots and scripts can now specify the location of a requirements.txt file which will be analyzed
     by the framework.
